from osgeo import gdal
import numpy as np
import math
import argparse
import logging

def convert_to_png(src_filename, out_filename):
  assert out_filename.endswith(".png")
  src_ds = gdal.Open(src_filename)
  gdal.Translate(out_filename, src_ds)
  print(f"Write to {out_filename}")

# normalize from [min_value, max_value] to [0.0, 1.0]
def normalize(value, min_value, max_value):
  return (value - min_value) / (max_value - min_value)

#
def encode_float32(data, min_val, max_val):
  #normalized_data = (normalize(data, min_val, max_val) * 65535.0).astype(np.uint16)
  #data_clean = np.nan_to_num(data, nan=0.0, posinf=0.0, neginf=0.0)

  data = np.clip(data, min_val, max_val)

  normalized_data = (data).astype(np.uint16)
  hi_byte = (normalized_data >> 8).astype(np.uint8)
  lo_byte = (normalized_data & 0xFF).astype(np.uint8)
  return [hi_byte, lo_byte]

def main(src_filename, out_filename):

  print(f"in={src_filename}, out={out_filename}")
  # original data is encoded in single band
  src_ds = gdal.Open(src_filename)

  if src_ds == None:
    print(f"Could not open {src_filename}")
    exit(1)

  x_size = src_ds.RasterXSize
  y_size = src_ds.RasterYSize

  print(f"x_size={x_size}, y_size={y_size}")

  src_band = src_ds.GetRasterBand(1)

  # get min and max value
  (min_val, max_val) = src_band.ComputeRasterMinMax(1)
  # print(f"min={min_val}, max={max_val}")

  driver = gdal.GetDriverByName('GTiff')
  out_ds = driver.Create(utf8_path=out_filename, xsize=src_ds.RasterXSize, ysize=src_ds.RasterYSize, bands=3, eType=gdal.GDT_Byte)
  out_ds.SetGeoTransform(src_ds.GetGeoTransform())
  out_ds.SetProjection(src_ds.GetProjection())

  # TODO: calculate size of chunks
  chunk_size = 100

  x_chunk_count = math.ceil(x_size / chunk_size)
  y_chunk_count = math.ceil(y_size / chunk_size)

  print(f"x_chunk_count={x_chunk_count}, y_chunk_count={y_chunk_count}")

  x_size_left = x_size
  x_offset = 0

  y_size_left = y_size
  y_offset = 0

  # read and parse in chunks
  for y in range(y_chunk_count):

    y_read_size = min(chunk_size, y_size_left )

    print(f"xoff={0}, yoff={y_offset}/{y_size}, xsize={x_size}, ysize={y_read_size}")

    # read and process
    data = src_band.ReadAsArray(0,y_offset,x_size,y_read_size).astype(np.float32)

    # encode float32 as two uint8 values
    [high_byte, low_byte] = encode_float32(data, min_val, max_val)

    out_band1 = out_ds.GetRasterBand(1)
    out_band1.WriteArray(high_byte, 0, y_offset)
    out_band1.SetNoDataValue(0)

    out_band2 = out_ds.GetRasterBand(2)
    out_band2.WriteArray(low_byte, 0, y_offset)
    out_band2.SetNoDataValue(0)

    y_size_left -= chunk_size
    y_offset += chunk_size


  out_band1.FlushCache()
  out_band2.FlushCache()

  del out_ds
  del src_ds

  print(f"Write to '{out_filename}'")

# Encode heightmap from float32 to uint8, using the red and green channels
# to improve accuracy.
if __name__ == "__main__":
  gdal.DontUseExceptions()

  parser = argparse.ArgumentParser()
  parser.add_argument("-s", "--src", type=str, help="Source file", required=True)
  parser.add_argument("-o", "--dst", type=str, help="Destination file", required=True)
  parser.add_argument("-p", "--png", help="Create PNG", action="store_true")
  args = parser.parse_args()

  src = args.src
  dst = args.dst


  main(src, dst)

  if args.png:
    convert_to_png(dst, f"{dst}.png")

